Package: ccfindR
Version: 1.31.0
Date: 2019-10-1
Title: Cancer Clone Finder
Authors@R: c(person('Jun','Woo',role=c('aut','cre'),email='jwoo@umn.edu'),
             person('Jinhua','Wang',role='aut',email='wangjh@umn.edu'))
Depends: R (>= 3.6.0)
Imports: stats, S4Vectors, utils, methods, Matrix,
        SummarizedExperiment, SingleCellExperiment, Rtsne, graphics,
        grDevices, gtools, RColorBrewer, ape, Rmpi, irlba, Rcpp, Rdpack
        (>= 0.7)
RdMacros: Rdpack
biocViews: Transcriptomics, SingleCell, ImmunoOncology, Bayesian,
        Clustering
Description: A collection of tools for cancer genomic data clustering
    analyses, including those for single cell RNA-seq.
    Cell clustering and feature gene selection analysis employ Bayesian 
    (and maximum likelihood) non-negative matrix factorization (NMF) algorithm. 
    Input data set consists of RNA count matrix, gene, and cell bar code 
    annotations.  Analysis outputs are factor matrices for multiple ranks and
    marginal likelihood values for each rank. The package includes utilities for 
    downstream analyses, including meta-gene identification, visualization, and 
    construction of rank-based trees for clusters.
License: GPL (>= 2)
NeedsCompilation: yes
URL: http://dx.doi.org/10.26508/lsa.201900443
RoxygenNote: 6.1.1
Encoding: UTF-8
Suggests: BiocStyle, knitr, rmarkdown
VignetteBuilder: knitr
LinkingTo: Rcpp, RcppEigen
Author: Jun Woo [aut, cre],
  Jinhua Wang [aut]
Maintainer: Jun Woo <jwoo@umn.edu>
git_url: https://git.bioconductor.org/packages/ccfindR
git_branch: devel
git_last_commit: ee2b32c
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-11-03
Packaged: 2025-11-03 21:40:07 UTC; biocbuild
