# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

vbnmf_update <- function(X, wh, hyper, fudge) {
    .Call(`_ccfindR_vbnmf_update`, X, wh, hyper, fudge)
}

