#' ccfindR: Cancer Clone FindeR
#' 
#' This package contains tools and utilities for cell-type discovery
#' using single-cell transcriptomic data while evaluating significance
#' of the depth of clustering \insertCite{woo_etal}{ccfindR}.
#' @references
#' \insertRef{woo_etal}{ccfindR}
#' 
#' @docType package
#' @name ccfindR
NULL  
