% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scNMF_class.R
\docType{methods}
\name{rowData,scNMFSet-method}
\alias{rowData,scNMFSet-method}
\title{Feature annotation accessor}
\usage{
\S4method{rowData}{scNMFSet}(x)
}
\arguments{
\item{x}{Object containing data}
}
\value{
DataFrame of feature annotation
}
\description{
Feature annotation accessor
}
\examples{
x <- matrix(rpois(n=12,lambda=3),4,3)
rownames(x) <- seq_len(4)
colnames(x) <- seq_len(3)
s <- scNMFSet(count=x,rowData=seq_len(4),colData=seq_len(3))
rowData(s) 
}
