% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scNMF_class.R
\name{basis}
\alias{basis}
\title{Basis matrices in an Object}
\usage{
basis(object)
}
\arguments{
\item{object}{Object of class \code{scNMFSet}}
}
\value{
Either \code{NULL} or a list of same length as 
\code{ranks(object)}, whose
elements are basis matrices derived from factorization 
under each rank value.
}
\description{
Retrieve or set the basis matrices \code{W} from factorization 
in an object
}
\details{
After factorization, basis matrices corresponding to each rank value are
stored as elements of a list, which is in slot \code{basis} of object of
class \code{scNMFSet}.
\code{basis(object)} will return the list of matrices. 
\code{basis(object) <- value}
can be used to modify it.
}
\examples{
s <- scNMFSet(count=matrix(rpois(n=12,lambda=3),4,3))
s <- vb_factorize(s,ranks=seq(2,4))
basis(s)[[1]]
}
